from SystemUpdater.Core.GenericConfigParser import Gconfig
from SystemUpdater.Core.PluginManager import pluginManagerClass
import logging
import json
import logging
DEFAULT_SETTINGS = "/etc/kylin-system-updater/settings-default.json"
RUN_SETTINGS = "/var/cache/kylin-system-updater/settings.json"
class JSONConfig:
    def __init__(self, settings_files = []):
        self.configs = {}

        # 循环读取所有的配置文件，并将配置进行合并，最后的配置文件优先级最高
        for config_file in settings_files:
            try:
                with open(config_file, "r") as f:
                    config_data = json.load(f)
                    self.configs = {**self.configs, **config_data}  # 使用字典解包进行合并
            except Exception as e:
                pass

        self.settings_files = settings_files
        self.skip_log = ["history", "update_history","system"]

    def get(self, key, default=None):
        return self.configs.get(key, default)

    def set(self, key, value):
        self.configs[key] = value

    def __getitem__(self, key):
        return self.configs[key]

    def __setitem__(self, key, value):
        if key not in self.skip_log:
            logging.info(f"Settings: setting '{key}' to '{value}'")
        self.configs[key] = value
        self.save()

    def update(self, new_data):
        if isinstance(new_data, dict):
            self.configs.update(new_data)
        else:
            raise TypeError("update() expects a dictionary")
        
    def save(self):
        with open(self.settings_files[-1], 'w') as f:
            json.dump(self.configs, f, indent=4)

"""
 使用方式：
    获取值
    1、settings.get("name")  == "Bob"
    2、settings.get("name1", "default") == "default"
    3、settings.get("local",{}).get("pre",{}).get("en") == "xxxxx"

    设置值
    1、settings.set("name", "Bob")
    # 仅支持一级设置
    2、settings["name"] = "Bob"
    3、支持多级设置
        a、locals = settings.get("local",{})
        b、locals["pre"]["en"] = "xxxxx"

    保存
    settings.save()
"""

settings = JSONConfig(
    settings_files=[
        DEFAULT_SETTINGS,
        RUN_SETTINGS,
    ]
)

def check_pre_update_scripts():
    try:
        # 获取检查更新脚本是否已经执行，执行过时不需要重复执行，包括更新程序自升级完成后、其他应用等等
        executed = settings.get("pre_update_executed",False)
        if executed:
            logging.info("pre_update_scripts has executed and skip...")
        else:
            genericConfig = Gconfig()
            # 获取 script 字段，若不存在则返回空列表
            scripts = genericConfig.get("script", {})
            pre_update_scripts_list = scripts.get("pre_update_script",[])
            logging.info(f"pre_update scripts list is {pre_update_scripts_list}")
            for srcipt in pre_update_scripts_list:
                pluginManagerClass().run_plugin(srcipt)
            
            # 设置已执行并保存到配置文件中
            settings["pre_update_executed"] = True
    except Exception as e:
        logging.error("pre_update_scripts exec failed")
        logging.error(e)

def check_post_update_scripts():
    try:
        # 更新后脚本不需要判断是否已经执行，由于更新后脚本仅可能执行一遍，更新前脚本会递归调用造成重复执行的问题
        genericConfig = Gconfig()
        # 获取 script 字段，若不存在则返回空列表
        scripts = genericConfig.get("script", {})
        post_update_scripts_list = scripts.get("post_update_script",[])
        logging.info(f"pre_update scripts list is {post_update_scripts_list}")
        for srcipt in post_update_scripts_list:
            pluginManagerClass().run_plugin(srcipt)

        # 设置执行流程执行完毕，并保存到配置文件中
        settings["pre_update_executed"] = False
    except Exception as e:
        logging.error("post_update_scripts exec failed")
        logging.error(e)