import os
import ctypes
import logging
import subprocess

from enum import Enum


class UpdaterSDKInterface:
    def __init__(self, libraryPath = '', archRelatedLib = ''):

        self.sdkLib     = None
        self.sdkLoaded  = False

        try:
            if archRelatedLib:
                libraryPath = self.arch_related_library_path(archRelatedLib)

            if os.path.exists(libraryPath):
                self.sdkLib     = ctypes.cdll.LoadLibrary(libraryPath)
                self.sdkLoaded  = True
            else:
                logging.warning("%s dose not exist.", libraryPath)
        except Exception as e:
            logging.error("UpdaterSDKInterface init error: %s", e)

    def arch_related_library_path(self, archRelatedLib):
        archRelatedLibPath = ''

        try:
            multiarch = subprocess.check_output(['dpkg-architecture', '-qDEB_HOST_MULTIARCH'], timeout=3).decode().strip()
            archRelatedLibPath = os.path.join("/usr/lib", multiarch, archRelatedLib)
        except Exception as e:
            logging.error("arch_related_library_path error: %s", e)

        return archRelatedLibPath

    def function_exists(self, funcName):
        try:
            getattr(self.sdkLib, funcName)
            return True
        except AttributeError:
            return False

class UpdaterSDKSysInfo:
    SDK_LIBRARY = 'libkysysinfo.so'

    def __init__(self):
        self.sysInfoLoaded    = False
        self.sdkInterface     = UpdaterSDKInterface(archRelatedLib = self.SDK_LIBRARY)

        if not self.sdkInterface.sdkLoaded:
            logging.warning("'%s' load error", self.SDK_LIBRARY)
        else:
            self.sysinfo_interface_init()

    def sysinfo_interface_init(self):
        try:
            if self.sdkInterface.function_exists("kdk_system_get_appScene"):
                self.sdkInterface.sdkLib.kdk_system_get_appScene.restype   = ctypes.c_char_p
            else:
                logging.warning("kdk_system_get_appScene do not find in '%s'", self.SDK_LIBRARY)
                return

            self.sysInfoLoaded = True
        except Exception as e:
            logging.error("package_interface_init error: %s", e)

    def sysinfo_interface_get_appScen(self):
        _sysinfo = ''

        try:
            if not self.sysInfoLoaded:
                logging.error("sysinfo_interface_get_appScen load library error")
                return _sysinfo

            _sysinfo = self.sdkInterface.sdkLib.kdk_system_get_appScene().decode('utf-8').strip()

            logging.info("sysinfo_interface_get_appScen _sysinfo: %s", _sysinfo)

        except Exception as e:
            logging.error("sysinfo_interface_get_appScen error: %s", e)

        return _sysinfo

    